package api

import (
	"encoding/json"
	"net/http"
	"witness/core"

	"github.com/ethereum/go-ethereum/common"
	geth "github.com/ethereum/go-ethereum/mobile"
	log "github.com/sirupsen/logrus"
)

var witness *core.Witness

func rpcHandle(w http.ResponseWriter, r *http.Request) {
	req := &jsonrpcMessage{}
	resp := &jsonrpcMessage{}

	if r.Method != "POST" {
		w.Write([]byte("method not allowed"))
		w.WriteHeader(http.StatusMethodNotAllowed)
		return
	}

	w.Header().Set("Content-Type", "application/json")
	contentType := r.Header.Get("Content-Type")
	if contentType != "application/json" {
		resp.Error = &jsonError{
			Code:    -32600,
			Message: "invalid Content-Type header",
		}
		resp.Version = "2.0"
		_ = json.NewEncoder(w).Encode(resp)
		return
	}

	err := json.NewDecoder(r.Body).Decode(req)
	if err != nil {
		resp.Error = &jsonError{
			Code:    -32603,
			Message: err.Error(),
		}
		resp.Version = "2.0"
		_ = json.NewEncoder(w).Encode(resp)
		return
	}

	if req.Version != "2.0" {
		resp.Error = &jsonError{
			Code:    -32600,
			Message: "invalid jsonrpc version",
		}
		resp.Version = "2.0"
		_ = json.NewEncoder(w).Encode(resp)
		return
	}
	resp.Version = req.Version
	resp.ID = req.ID

	switch req.Method {
	case "getWithdrawProofs":
		getWithdrawProofs(req.Params, resp)
		_ = json.NewEncoder(w).Encode(resp)
		return
	case "getPendingWorkload":
		getPendingWorkload(req.Params, resp)
		_ = json.NewEncoder(w).Encode(resp)
		return

	default:
		resp.Error = &jsonError{
			Code:    -32601,
			Message: "method not found",
		}
		_ = json.NewEncoder(w).Encode(resp)
		return
	}

}

func getWithdrawProofs(params []byte, resp *jsonrpcMessage) {
	addressList := make([]string, 0)
	err := json.Unmarshal(params, &addressList)
	if err != nil || len(addressList) != 1 {
		resp.Error = &jsonError{
			Code:    -32602,
			Message: "invalid params",
		}
		return
	}

	_, err = geth.NewAddressFromHex(addressList[0])
	if err != nil {
		resp.Error = &jsonError{
			Code:    -32602,
			Message: "invalid params",
		}
		return
	}

	workload, proofs := witness.GetMerkleProof(common.HexToAddress(addressList[0]))

	temp := map[string]interface{}{
		"workload": workload,
		"proofs":   proofs,
	}

	resp.Result, _ = json.Marshal(temp)
	return
}

func getPendingWorkload(params []byte, resp *jsonrpcMessage) {
	addressList := make([]string, 0)
	err := json.Unmarshal(params, &addressList)
	if err != nil || len(addressList) != 1 {
		resp.Error = &jsonError{
			Code:    -32602,
			Message: "invalid params",
		}
		return
	}

	_, err = geth.NewAddressFromHex(addressList[0])
	if err != nil {
		resp.Error = &jsonError{
			Code:    -32602,
			Message: "invalid params",
		}
		return
	}

	workload, globalWorkload := witness.GetPendingWorkload(common.HexToAddress(addressList[0]))
	temp := map[string]interface{}{
		"workload":        workload,
		"global_workload": globalWorkload,
	}

	resp.Result, _ = json.Marshal(temp)
	return
}

func StartJSONRPC(listenAddress string, w *core.Witness) {
	witness = w
	http.HandleFunc("/", rpcHandle)
	log.WithField("listen", listenAddress).Info("start JSON-RPC server")
	err := http.ListenAndServe(listenAddress, nil)
	if err != nil {
		log.WithError(err).Fatal("failed to start JSON-RPC server")
	}
}
